import urllib.request
import urllib.parse
import urllib.error
import http.cookiejar
from html.parser import HTMLParser


class ATError(Exception):
    def __init__(self, error_msg):
        super().__init__(self)
        self.error_msg = error_msg

    def __str__(self):
        return self.error_msg


class TokenParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.token = None

    def handle_starttag(self, tag, attrs):
        if tag == 'input':
            if len(attrs) == 0:
                pass
            else:
                if ('name', 'token') in attrs and ('type', 'hidden') in attrs:
                    self.token = attrs[2][1]


class ContentParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.content = ''
        self.trigger = False
    
    def handle_starttag(self, tag, attrs):
        if tag != 'textarea':
            return
        if ('id', 'mdeditor') in attrs and ('name', 'content') in attrs:
            self.trigger = True
            return
    
    def handle_endtag(self, tag):
        if tag == 'textarea' and self.trigger:
            self.trigger = False

    def handle_data(self, data):
        if self.trigger:
            self.content = data


token_parser = TokenParser()
content_parser = ContentParser()


class ReportSender:
    def __init__(self, username, password):
        # general cookie jar
        cj = http.cookiejar.CookieJar()
        handler = urllib.request.HTTPCookieProcessor(cj)
        opener = urllib.request.build_opener(handler)
        self.opener = opener
        
        self.login(username, password)

    def parse_token(self, res):
        token_parser.feed(res)
        token_parser.close()
        return token_parser.token

    def parse_content(self, res):
        content_parser.feed(res)
        content_parser.close()
        return content_parser.content

    def login(self, username, password):
        url = 'https://at.twtstudio.com/login'
        print('Loggin into %s...' % url)

        try:
            token_res = self.opener.open(url)
            token = self.parse_token(token_res.read().decode('utf-8'))
            if not token:
                print('Failed to parse token from %s. Aborting' % url)
                raise ATError('token not found')
            data = {
                'token': token,
                'username': username,
                'password': password
            }
            post_data = urllib.parse.urlencode(data).encode('utf-8')
            login_req = urllib.request.Request(url, data=post_data, method='POST')
            self.opener.open(login_req)
        except Exception as e:
            raise ATError(str(e))

    def get_content(self):
        url = 'https://at.twtstudio.com/report/write'
        print('Fetching present weekly report...')
        try:
            request = urllib.request.Request(url)
            response = self.opener.open(request).read().decode('utf-8')
            write_token = self.parse_token(response)
        except Exception as e:
            raise ATError(str(e))
        
        if not write_token:
            print('Failed to parse token from %s. Aborting...' % url)
            raise ATError('token not found')
        old_content = self.parse_content(response)
        self.old_content = old_content
        self.write_token = write_token

    def write(self, content):
        url = 'https://at.twtstudio.com/report'
        print('Writing report with your commit messages...')

        data = {
            'token': self.write_token,
            'content': content
        }
        post_data = urllib.parse.urlencode(data).encode('utf-8')
        request = urllib.request.Request(url, data=post_data, method='POST')
        try:
            self.opener.open(request)
        except Exception as e:
            raise ATError(str(e))
    


import sys
import subprocess
import getopt
import re
from sender import ReportSender
from sender import ATError

def get_params(argv):
    display_name = ''
    username = ''
    password = ''
    try:
        opts, args = getopt.getopt(
            argv, 
            'hd:u:p:', 
            ['help', 'display-name=', 'username=', 'password=', 'thanks']
        )
    except getopt.GetoptError:
        print('Unknown option')
        print('Type install-auto-reporter --help for more information.')

    for opt, arg in opts:
        if opt in ('-h', '--help'):
            print('Usage:')
            sys.exit(0)
        if opt in ('-d', '--display-name'):
            display_name = arg
        if opt in ('-u', '--username'):
            username = arg
        if opt in ('-p', '--password'):
            password = arg
        if opt == '--thanks':
            print('mua')
            sys.exit(0)

    return (display_name, username, password)

def get_commit_msgs():
    for line in sys.stdin:
        local_ref, local_sha1, remote_ref, remote_sha1 = line.strip().split(' ')
        raw_messages = subprocess.check_output(
            ['git', 'show', '--format=%s', '-s', "%s..%s" % (remote_sha1, local_sha1)]
        )
        return raw_messages.decode('utf-8').splitlines()
    return None

def generate_report(report_array, title, body):
    new_report_array = report_array[:]
    if len(report_array) == 1:
        # no title found
        # append title
        new_report_array.append('\r\n%s' % title)
        new_report_array.append(body)
        return new_report_array
    elif len(report_array) == 2:
        # found title
        new_report_array.insert(1, title)
        new_report_array.insert(2, body)
        return new_report_array
    else:
        raise ATError('More than one title found in old report')

def main(argv, dir_name):
    display_name, username, password = get_params(argv)
    if not username or not password:
        print('username and password must be specificated.')
    if not display_name:
        display_name = dir_name
    commit_msgs = get_commit_msgs()
    if not commit_msgs or len(commit_msgs) == 0:
        print('Nothing to push. Did you commit before pushing?')
    
    try:
        title = '## %s\r\n' % display_name
        AT = ReportSender(username, password)
        AT.get_content()
        report_array = re.split(r'## %s\r\n' % display_name, AT.old_content)

        body_string = ''
        for commit_msg in commit_msgs:
            body_string = body_string + '- %s\r\n' % commit_msg

        new_report_array = generate_report(report_array, title, body_string)
        new_report = ''.join(new_report_array)
        AT.write(new_report)
        print('Success!')
    except ATError as e:
        print('Something bad happens:\n%s' % e)
        print('But you can still trying to push your commit to your remote server.')
        print('Notice: If you choose to continue, auto-reporter will NOT recive these commits next time, you\'ll have to manually add them to your report.')
        continue_push = input('Continue? [Y/n] ').lower()
        valid = {
            '': True,
            'y': True,
            'ye': True,
            'yes': True,
            'n': False,
            'no': False
        }
        if not valid[continue_push]:
            print('Aborting...')
            return 1
        return 0


import os
import stat
import subprocess
import sys

def main():
    git_dir = subprocess.check_output(['git', 'rev-parse', '--git-dir'])
    hook_file_path = os.path.join(
        os.path.abspath(git_dir.strip().decode('utf-8')), 'hooks', 'pre-push'
    )

    with open(hook_file_path, 'w') as fd:
        content = '%s -c "import sys; import os; import auto-reporter.hook; sys.exit(auto-reporter.main(%s, os.path.split(os.getcwd())[1]))" $@\n' % (sys.executable, sys.argv[1:])
        fd.write(content)
    os.chmod(hook_file_path, stat.S_IRWXU | stat.S_IRGRP | stat.S_IROTH)

    sys.stdout.write('Auto report installed.\n')
